@@ -1,28 +1,18 @@
-/********************************************************************
-Copyright 2014  Martin Gräßlin <mgraesslin@kde.org>
+/*
+    SPDX-FileCopyrightText: 2014 Martin Gräßlin <mgraesslin@kde.org>
 
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) version 3, or any
-later version accepted by the membership of KDE e.V. (or its
-successor approved by the membership of KDE e.V.), which shall
-act as a proxy defined in Section 6 of version 3 of the license.
-
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public
-License along with this library.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
 // Qt
-#include <QtTest/QtTest>
+#include <QFileSystemWatcher>
+#include <QProcess>
+#include <QSignalSpy>
+#include <QStandardPaths>
+#include <QTest>
 // KWin
 #include "../../src/client/connection_thread.h"
-#include "../../src/client/registry.h"
 #include "../../src/client/fullscreen_shell.h"
+#include "../../src/client/registry.h"
 // Wayland
 #include <wayland-client-protocol.h>
 
@@ -57,11 +47,12 @@ void TestWaylandFullscreenShell::init()
     QVERIFY(!m_westonProcess);
     // starts weston
     m_westonProcess = new QProcess(this);
-    m_westonProcess->setProgram(QStringLiteral("weston"));
+    const QString exec = QStandardPaths::findExecutable(QStringLiteral("weston"));
+    QVERIFY(!exec.isEmpty());
+    m_westonProcess->setProgram(exec);
 
-    m_westonProcess->setArguments(QStringList({QStringLiteral("--socket=%1").arg(s_socketName),
-                                               QStringLiteral("--backend=headless-backend.so"),
-                                               QStringLiteral("--shell=fullscreen-shell.so")}));
+    m_westonProcess->setArguments(QStringList(
+        {QStringLiteral("--socket=%1").arg(s_socketName), QStringLiteral("--backend=headless-backend.so"), QStringLiteral("--shell=fullscreen-shell.so")}));
     m_westonProcess->start();
     QVERIFY(m_westonProcess->waitForStarted());
     QTest::qWait(500);
@@ -72,7 +63,7 @@ void TestWaylandFullscreenShell::init()
         return;
     }
     QFileSystemWatcher *socketWatcher = new QFileSystemWatcher(QStringList({runtimeDir.absolutePath()}), this);
-    QSignalSpy socketSpy(socketWatcher, SIGNAL(directoryChanged(QString)));
+    QSignalSpy socketSpy(socketWatcher, &QFileSystemWatcher::directoryChanged);
 
     // limit to maximum of 10 waits
     for (int i = 0; i < 10; ++i) {
@@ -99,18 +90,25 @@ void TestWaylandFullscreenShell::testRegistry()
         QSKIP("This test requires a running wayland server");
     }
     KWayland::Client::ConnectionThread connection;
-    QSignalSpy connectedSpy(&connection, SIGNAL(connected()));
+    QSignalSpy connectedSpy(&connection, &KWayland::Client::ConnectionThread::connected);
     connection.setSocketName(s_socketName);
     connection.initConnection();
     QVERIFY(connectedSpy.wait());
 
     KWayland::Client::Registry registry;
-    QSignalSpy announced(&registry, SIGNAL(fullscreenShellAnnounced(quint32,quint32)));
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfaceAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
+    QSignalSpy announced(&registry, &KWayland::Client::Registry::fullscreenShellAnnounced);
     registry.create(connection.display());
     QVERIFY(registry.isValid());
     registry.setup();
     wl_display_flush(connection.display());
-    QVERIFY(announced.wait());
+    QVERIFY(interfacesAnnouncedSpy.wait());
+
+    if (!registry.hasInterface(KWayland::Client::Registry::Interface::FullscreenShell)) {
+        QSKIP("Weston does not have fullscreen shell support");
+    }
+    QCOMPARE(announced.count(), 1);
 
     KWayland::Client::FullscreenShell fullscreenShell;
     QVERIFY(!fullscreenShell.isValid());
@@ -122,22 +120,30 @@ void TestWaylandFullscreenShell::testRegistry()
 }
 
 void TestWaylandFullscreenShell::testRegistryCreate()
-{    if (m_westonProcess->state() != QProcess::Running) {
+{
+    if (m_westonProcess->state() != QProcess::Running) {
         QSKIP("This test requires a running wayland server");
     }
     KWayland::Client::ConnectionThread connection;
-    QSignalSpy connectedSpy(&connection, SIGNAL(connected()));
+    QSignalSpy connectedSpy(&connection, &KWayland::Client::ConnectionThread::connected);
     connection.setSocketName(s_socketName);
     connection.initConnection();
     QVERIFY(connectedSpy.wait());
 
     KWayland::Client::Registry registry;
-    QSignalSpy announced(&registry, SIGNAL(fullscreenShellAnnounced(quint32,quint32)));
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfaceAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
+    QSignalSpy announced(&registry, &KWayland::Client::Registry::fullscreenShellAnnounced);
     registry.create(connection.display());
     QVERIFY(registry.isValid());
     registry.setup();
     wl_display_flush(connection.display());
-    QVERIFY(announced.wait());
+    QVERIFY(interfacesAnnouncedSpy.wait());
+
+    if (!registry.hasInterface(KWayland::Client::Registry::Interface::FullscreenShell)) {
+        QSKIP("Weston does not have fullscreen shell support");
+    }
+    QCOMPARE(announced.count(), 1);
 
     KWayland::Client::FullscreenShell *fullscreenShell = registry.createFullscreenShell(announced.first().first().value<quint32>(), 1, &registry);
     QVERIFY(fullscreenShell->isValid());
