@@ -1,25 +1,12 @@
-/********************************************************************
-Copyright 2014 Martin Gräßlin <mgraesslin@kde.org>
-Copyright 2015 Sebastian Kügler <sebas@kde.org>
+/*
+    SPDX-FileCopyrightText: 2014 Martin Gräßlin <mgraesslin@kde.org>
+    SPDX-FileCopyrightText: 2015 Sebastian Kügler <sebas@kde.org>
 
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) version 3, or any
-later version accepted by the membership of KDE e.V. (or its
-successor approved by the membership of KDE e.V.), which shall
-act as a proxy defined in Section 6 of version 3 of the license.
-
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public
-License along with this library.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
 // Qt
-#include <QtTest/QtTest>
+#include <QSignalSpy>
+#include <QTest>
 // KWin
 #include "../../src/client/connection_thread.h"
 #include "../../src/client/event_queue.h"
@@ -44,7 +31,11 @@ private Q_SLOTS:
 
     void testRegistry();
     void testModeChanges();
+#if KWAYLANDSERVER_ENABLE_DEPRECATED_SINCE(5, 50)
+    void testScaleChange_legacy();
+#endif
     void testScaleChange();
+    void testColorCurvesChange();
 
     void testSubPixel_data();
     void testSubPixel();
@@ -61,10 +52,13 @@ private:
     KWayland::Server::Display *m_display;
     KWayland::Server::OutputDeviceInterface *m_serverOutputDevice;
     QByteArray m_edid;
+    QString m_serialNumber;
+    QString m_eidaId;
+
+    KWayland::Server::OutputDeviceInterface::ColorCurves m_initColorCurves;
     KWayland::Client::ConnectionThread *m_connection;
     KWayland::Client::EventQueue *m_queue;
     QThread *m_thread;
-
 };
 
 static const QString s_socketName = QStringLiteral("kwin-test-wayland-output-0");
@@ -74,6 +68,7 @@ TestWaylandOutputDevice::TestWaylandOutputDevice(QObject *parent)
     , m_display(nullptr)
     , m_serverOutputDevice(nullptr)
     , m_connection(nullptr)
+    , m_queue(nullptr)
     , m_thread(nullptr)
 {
 }
@@ -90,7 +85,6 @@ void TestWaylandOutputDevice::init()
     m_serverOutputDevice = m_display->createOutputDevice(this);
     m_serverOutputDevice->setUuid("1337");
 
-
     OutputDeviceInterface::Mode m0;
     m0.id = 0;
     m0.size = QSize(800, 600);
@@ -110,9 +104,32 @@ void TestWaylandOutputDevice::init()
 
     m_serverOutputDevice->setCurrentMode(1);
 
-    m_edid = QByteArray::fromBase64("AP///////wAQrBbwTExLQQ4WAQOANCB46h7Frk80sSYOUFSlSwCBgKlA0QBxTwEBAQEBAQEBKDyAoHCwI0AwIDYABkQhAAAaAAAA/wBGNTI1TTI0NUFLTEwKAAAA/ABERUxMIFUyNDEwCiAgAAAA/QA4TB5REQAKICAgICAgAToCAynxUJAFBAMCBxYBHxITFCAVEQYjCQcHZwMMABAAOC2DAQAA4wUDAQI6gBhxOC1AWCxFAAZEIQAAHgEdgBhxHBYgWCwlAAZEIQAAngEdAHJR0B4gbihVAAZEIQAAHowK0Iog4C0QED6WAAZEIQAAGAAAAAAAAAAAAAAAAAAAPg==");
+    m_edid = QByteArray::fromBase64(
+        "AP///////wAQrBbwTExLQQ4WAQOANCB46h7Frk80sSYOUFSlSwCBgKlA0QBxTwEBAQEBAQEBKDyAoHCwI0AwIDYABkQhAAAaAAAA/wBGNTI1TTI0NUFLTEwKAAAA/ABERUxMIFUyNDEwCiAgAAAA/"
+        "QA4TB5REQAKICAgICAgAToCAynxUJAFBAMCBxYBHxITFCAVEQYjCQcHZwMMABAAOC2DAQAA4wUDAQI6gBhxOC1AWCxFAAZEIQAAHgEdgBhxHBYgWCwlAAZEIQAAngEdAHJR0B4gbihVAAZEIQAAHow"
+        "K0Iog4C0QED6WAAZEIQAAGAAAAAAAAAAAAAAAAAAAPg==");
     m_serverOutputDevice->setEdid(m_edid);
 
+    m_serialNumber = "23498723948723";
+    m_serverOutputDevice->setSerialNumber(m_serialNumber);
+    m_eidaId = "asdffoo";
+    m_serverOutputDevice->setEisaId(m_eidaId);
+
+    m_initColorCurves.red.clear();
+    m_initColorCurves.green.clear();
+    m_initColorCurves.blue.clear();
+    // 8 bit color ramps
+    for (int i = 0; i < 256; i++) {
+        quint16 val = (double)i / 255 * UINT16_MAX;
+        m_initColorCurves.red << val;
+        m_initColorCurves.green << val;
+    }
+    // 10 bit color ramp
+    for (int i = 0; i < 320; i++) {
+        m_initColorCurves.blue << (double)i / 319 * UINT16_MAX;
+    }
+    m_serverOutputDevice->setColorCurves(m_initColorCurves);
+
     m_serverOutputDevice->create();
 
     // setup connection
@@ -161,12 +178,14 @@ void TestWaylandOutputDevice::testRegistry()
     m_serverOutputDevice->setPhysicalSize(QSize(200, 100));
 
     KWayland::Client::Registry registry;
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfacesAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
     QSignalSpy announced(&registry, &KWayland::Client::Registry::outputDeviceAnnounced);
     registry.create(m_connection->display());
     QVERIFY(registry.isValid());
     registry.setup();
     wl_display_flush(m_connection->display());
-    QVERIFY(announced.wait());
+    QVERIFY(interfacesAnnouncedSpy.wait());
 
     KWayland::Client::OutputDevice output;
     QVERIFY(!output.isValid());
@@ -178,11 +197,20 @@ void TestWaylandOutputDevice::testRegistry()
     QCOMPARE(output.physicalSize(), QSize());
     QCOMPARE(output.pixelSize(), QSize());
     QCOMPARE(output.refreshRate(), 0);
+#if KWAYLANDSERVER_ENABLE_DEPRECATED_SINCE(5, 50)
     QCOMPARE(output.scale(), 1);
+#endif
+    QCOMPARE(output.scaleF(), 1.0);
+    QCOMPARE(output.colorCurves().red, QVector<quint16>());
+    QCOMPARE(output.colorCurves().green, QVector<quint16>());
+    QCOMPARE(output.colorCurves().blue, QVector<quint16>());
     QCOMPARE(output.subPixel(), KWayland::Client::OutputDevice::SubPixel::Unknown);
     QCOMPARE(output.transform(), KWayland::Client::OutputDevice::Transform::Normal);
     QCOMPARE(output.enabled(), OutputDevice::Enablement::Enabled);
     QCOMPARE(output.edid(), QByteArray());
+    QCOMPARE(output.eisaId(), QString());
+    QCOMPARE(output.serialNumber(), QString());
+
     QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::done);
     QVERIFY(outputChanged.isValid());
 
@@ -198,7 +226,13 @@ void TestWaylandOutputDevice::testRegistry()
     QCOMPARE(output.physicalSize(), QSize(200, 100));
     QCOMPARE(output.pixelSize(), QSize(1024, 768));
     QCOMPARE(output.refreshRate(), 60000);
+#if KWAYLANDSERVER_ENABLE_DEPRECATED_SINCE(5, 50)
     QCOMPARE(output.scale(), 1);
+#endif
+    QCOMPARE(output.scaleF(), 1.0);
+    QCOMPARE(output.colorCurves().red, m_initColorCurves.red);
+    QCOMPARE(output.colorCurves().green, m_initColorCurves.green);
+    QCOMPARE(output.colorCurves().blue, m_initColorCurves.blue);
     // for xwayland output it's unknown
     QCOMPARE(output.subPixel(), KWayland::Client::OutputDevice::SubPixel::Unknown);
     // for xwayland transform is normal
@@ -207,29 +241,34 @@ void TestWaylandOutputDevice::testRegistry()
     QCOMPARE(output.edid(), m_edid);
     QCOMPARE(output.enabled(), OutputDevice::Enablement::Enabled);
     QCOMPARE(output.uuid(), QByteArray("1337"));
-
+    QCOMPARE(output.serialNumber(), m_serialNumber);
+    QCOMPARE(output.eisaId(), m_eidaId);
 }
 
 void TestWaylandOutputDevice::testModeChanges()
 {
     using namespace KWayland::Client;
     KWayland::Client::Registry registry;
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfacesAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
     QSignalSpy announced(&registry, &KWayland::Client::Registry::outputDeviceAnnounced);
     registry.setEventQueue(m_queue);
     registry.create(m_connection->display());
     QVERIFY(registry.isValid());
     registry.setup();
     wl_display_flush(m_connection->display());
-    QVERIFY(announced.wait());
+    QVERIFY(interfacesAnnouncedSpy.wait());
 
     KWayland::Client::OutputDevice output;
     QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::changed);
     QVERIFY(outputChanged.isValid());
     QSignalSpy modeAddedSpy(&output, &KWayland::Client::OutputDevice::modeAdded);
     QVERIFY(modeAddedSpy.isValid());
+    QSignalSpy doneSpy(&output, &KWayland::Client::OutputDevice::done);
+    QVERIFY(doneSpy.isValid());
     output.setup(registry.bindOutputDevice(announced.first().first().value<quint32>(), announced.first().last().value<quint32>()));
     wl_display_flush(m_connection->display());
-    QVERIFY(outputChanged.wait());
+    QVERIFY(doneSpy.wait());
     QCOMPARE(modeAddedSpy.count(), 3);
 
     QCOMPARE(modeAddedSpy.at(0).first().value<OutputDevice::Mode>().size, QSize(800, 600));
@@ -262,10 +301,7 @@ void TestWaylandOutputDevice::testModeChanges()
     QSignalSpy modeChangedSpy(&output, &KWayland::Client::OutputDevice::modeChanged);
     QVERIFY(modeChangedSpy.isValid());
     m_serverOutputDevice->setCurrentMode(0);
-    QVERIFY(modeChangedSpy.wait());
-    if (modeChangedSpy.size() == 1) {
-        QVERIFY(modeChangedSpy.wait());
-    }
+    QVERIFY(doneSpy.wait());
     QCOMPARE(modeChangedSpy.size(), 2);
     // the one which lost the current flag
     QCOMPARE(modeChangedSpy.first().first().value<OutputDevice::Mode>().size, QSize(1024, 768));
@@ -274,7 +310,8 @@ void TestWaylandOutputDevice::testModeChanges()
     // the one which got the current flag
     QCOMPARE(modeChangedSpy.last().first().value<OutputDevice::Mode>().size, QSize(800, 600));
     QCOMPARE(modeChangedSpy.last().first().value<OutputDevice::Mode>().refreshRate, 60000);
-    QCOMPARE(modeChangedSpy.last().first().value<OutputDevice::Mode>().flags, OutputDevice::Mode::Flags(OutputDevice::Mode::Flag::Current | OutputDevice::Mode::Flag::Preferred));
+    QCOMPARE(modeChangedSpy.last().first().value<OutputDevice::Mode>().flags,
+             OutputDevice::Mode::Flags(OutputDevice::Mode::Flag::Current | OutputDevice::Mode::Flag::Preferred));
     QVERIFY(!outputChanged.isEmpty());
     QCOMPARE(output.pixelSize(), QSize(800, 600));
     const QList<OutputDevice::Mode> &modes2 = output.modes();
@@ -292,10 +329,7 @@ void TestWaylandOutputDevice::testModeChanges()
     outputChanged.clear();
     modeChangedSpy.clear();
     m_serverOutputDevice->setCurrentMode(2);
-    QVERIFY(modeChangedSpy.wait());
-    if (modeChangedSpy.size() == 1) {
-        QVERIFY(modeChangedSpy.wait());
-    }
+    QVERIFY(doneSpy.wait());
     QCOMPARE(modeChangedSpy.size(), 2);
     // the one which lost the current flag
     QCOMPARE(modeChangedSpy.first().first().value<OutputDevice::Mode>().size, QSize(800, 600));
@@ -309,35 +343,130 @@ void TestWaylandOutputDevice::testModeChanges()
     QCOMPARE(output.pixelSize(), QSize(1280, 1024));
 }
 
-void TestWaylandOutputDevice::testScaleChange()
+#if KWAYLANDSERVER_ENABLE_DEPRECATED_SINCE(5, 50)
+void TestWaylandOutputDevice::testScaleChange_legacy()
 {
     KWayland::Client::Registry registry;
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfacesAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
     QSignalSpy announced(&registry, &KWayland::Client::Registry::outputDeviceAnnounced);
+    registry.setEventQueue(m_queue);
     registry.create(m_connection->display());
     QVERIFY(registry.isValid());
     registry.setup();
     wl_display_flush(m_connection->display());
-    QVERIFY(announced.wait());
+    QVERIFY(interfacesAnnouncedSpy.wait());
 
     KWayland::Client::OutputDevice output;
-    QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::changed);
+    QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::done);
     QVERIFY(outputChanged.isValid());
     output.setup(registry.bindOutputDevice(announced.first().first().value<quint32>(), announced.first().last().value<quint32>()));
     wl_display_flush(m_connection->display());
     QVERIFY(outputChanged.wait());
     QCOMPARE(output.scale(), 1);
+    QCOMPARE(output.scaleF(), 1.0);
 
     // change the scale
     outputChanged.clear();
     m_serverOutputDevice->setScale(2);
     QVERIFY(outputChanged.wait());
     QCOMPARE(output.scale(), 2);
+    QCOMPARE(output.scaleF(), 2.0); // check we're forward compatible
 
     // change once more
     outputChanged.clear();
     m_serverOutputDevice->setScale(4);
     QVERIFY(outputChanged.wait());
     QCOMPARE(output.scale(), 4);
+    QCOMPARE(output.scaleF(), 4.0);
+}
+#endif
+
+void TestWaylandOutputDevice::testScaleChange()
+{
+    KWayland::Client::Registry registry;
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfacesAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
+    QSignalSpy announced(&registry, &KWayland::Client::Registry::outputDeviceAnnounced);
+    registry.setEventQueue(m_queue);
+    registry.create(m_connection->display());
+    QVERIFY(registry.isValid());
+    registry.setup();
+    wl_display_flush(m_connection->display());
+    QVERIFY(interfacesAnnouncedSpy.wait());
+
+    KWayland::Client::OutputDevice output;
+    QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::done);
+    QVERIFY(outputChanged.isValid());
+    output.setup(registry.bindOutputDevice(announced.first().first().value<quint32>(), announced.first().last().value<quint32>()));
+    wl_display_flush(m_connection->display());
+    QVERIFY(outputChanged.wait());
+    QCOMPARE(output.scaleF(), 1.0);
+
+    // change the scale
+    outputChanged.clear();
+    m_serverOutputDevice->setScaleF(2.2);
+    QVERIFY(outputChanged.wait());
+#if KWAYLANDSERVER_ENABLE_DEPRECATED_SINCE(5, 50)
+    QCOMPARE(output.scale(), 2); // check backwards compatibility works
+#endif
+    QCOMPARE(wl_fixed_from_double(output.scaleF()), wl_fixed_from_double(2.2));
+
+    // change once more
+    outputChanged.clear();
+    m_serverOutputDevice->setScaleF(4.9);
+    QVERIFY(outputChanged.wait());
+#if KWAYLANDSERVER_ENABLE_DEPRECATED_SINCE(5, 50)
+    QCOMPARE(output.scale(), 5);
+#endif
+    QCOMPARE(wl_fixed_from_double(output.scaleF()), wl_fixed_from_double(4.9));
+}
+
+void TestWaylandOutputDevice::testColorCurvesChange()
+{
+    KWayland::Client::Registry registry;
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfacesAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
+    QSignalSpy announced(&registry, &KWayland::Client::Registry::outputDeviceAnnounced);
+    registry.setEventQueue(m_queue);
+    registry.create(m_connection->display());
+    QVERIFY(registry.isValid());
+    registry.setup();
+    wl_display_flush(m_connection->display());
+    QVERIFY(interfacesAnnouncedSpy.wait());
+
+    KWayland::Client::OutputDevice output;
+    QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::done);
+    QVERIFY(outputChanged.isValid());
+    output.setup(registry.bindOutputDevice(announced.first().first().value<quint32>(), announced.first().last().value<quint32>()));
+    wl_display_flush(m_connection->display());
+    QVERIFY(outputChanged.wait());
+    QCOMPARE(output.colorCurves().red, m_initColorCurves.red);
+    QCOMPARE(output.colorCurves().green, m_initColorCurves.green);
+    QCOMPARE(output.colorCurves().blue, m_initColorCurves.blue);
+
+    // change the color curves
+    outputChanged.clear();
+    KWayland::Server::OutputDeviceInterface::ColorCurves cc;
+    cc.red = QVector<quint16>(256, 0);
+    cc.green = QVector<quint16>(256, UINT16_MAX);
+    cc.blue = QVector<quint16>(320, 1);
+    m_serverOutputDevice->setColorCurves(cc);
+    QVERIFY(outputChanged.wait());
+    QCOMPARE(output.colorCurves().red, cc.red);
+    QCOMPARE(output.colorCurves().green, cc.green);
+    QCOMPARE(output.colorCurves().blue, cc.blue);
+
+    // change once more
+    outputChanged.clear();
+    cc.red = QVector<quint16>(256, 0);
+    cc.green = QVector<quint16>(256, UINT16_MAX);
+    cc.blue = QVector<quint16>(320, UINT16_MAX);
+    m_serverOutputDevice->setColorCurves(cc);
+    QVERIFY(outputChanged.wait());
+    QCOMPARE(output.colorCurves().red, cc.red);
+    QCOMPARE(output.colorCurves().green, cc.green);
+    QCOMPARE(output.colorCurves().blue, cc.blue);
 }
 
 void TestWaylandOutputDevice::testSubPixel_data()
@@ -362,30 +491,29 @@ void TestWaylandOutputDevice::testSubPixel()
     m_serverOutputDevice->setSubPixel(actual);
 
     KWayland::Client::Registry registry;
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfacesAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
     QSignalSpy announced(&registry, &KWayland::Client::Registry::outputDeviceAnnounced);
+    registry.setEventQueue(m_queue);
     registry.create(m_connection->display());
     QVERIFY(registry.isValid());
     registry.setup();
     wl_display_flush(m_connection->display());
-    QVERIFY(announced.wait());
+    QVERIFY(interfacesAnnouncedSpy.wait());
 
     KWayland::Client::OutputDevice output;
-    QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::changed);
+    QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::done);
     QVERIFY(outputChanged.isValid());
     output.setup(registry.bindOutputDevice(announced.first().first().value<quint32>(), announced.first().last().value<quint32>()));
     wl_display_flush(m_connection->display());
-    if (outputChanged.isEmpty()) {
-        QVERIFY(outputChanged.wait());
-    }
+    QVERIFY(outputChanged.wait());
 
     QTEST(output.subPixel(), "expected");
 
     // change back to unknown
     outputChanged.clear();
     m_serverOutputDevice->setSubPixel(OutputDeviceInterface::SubPixel::Unknown);
-    if (outputChanged.isEmpty()) {
-        QVERIFY(outputChanged.wait());
-    }
+    QVERIFY(outputChanged.wait());
     QCOMPARE(output.subPixel(), OutputDevice::SubPixel::Unknown);
 }
 
@@ -396,13 +524,13 @@ void TestWaylandOutputDevice::testTransform_data()
     QTest::addColumn<KWayland::Client::OutputDevice::Transform>("expected");
     QTest::addColumn<KWayland::Server::OutputDeviceInterface::Transform>("actual");
 
+    // clang-format off
     QTest::newRow("90")          << OutputDevice::Transform::Rotated90  << OutputDeviceInterface::Transform::Rotated90;
     QTest::newRow("180")         << OutputDevice::Transform::Rotated180 << OutputDeviceInterface::Transform::Rotated180;
     QTest::newRow("270")         << OutputDevice::Transform::Rotated270 << OutputDeviceInterface::Transform::Rotated270;
     QTest::newRow("Flipped")     << OutputDevice::Transform::Flipped    << OutputDeviceInterface::Transform::Flipped;
     QTest::newRow("Flipped 90")  << OutputDevice::Transform::Flipped90  << OutputDeviceInterface::Transform::Flipped90;
-    QTest::newRow("Flipped 180") << OutputDevice::Transform::Flipped180 << OutputDeviceInterface::Transform::Flipped180;
-    QTest::newRow("Flipped 280") << OutputDevice::Transform::Flipped270 << OutputDeviceInterface::Transform::Flipped270;
+    // clang-format on
 }
 
 void TestWaylandOutputDevice::testTransform()
@@ -413,44 +541,47 @@ void TestWaylandOutputDevice::testTransform()
     m_serverOutputDevice->setTransform(actual);
 
     KWayland::Client::Registry registry;
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfacesAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
     QSignalSpy announced(&registry, &KWayland::Client::Registry::outputDeviceAnnounced);
+    registry.setEventQueue(m_queue);
     registry.create(m_connection->display());
     QVERIFY(registry.isValid());
     registry.setup();
     wl_display_flush(m_connection->display());
-    QVERIFY(announced.wait());
+    QVERIFY(interfacesAnnouncedSpy.wait());
 
-    KWayland::Client::OutputDevice *output = registry.createOutputDevice(announced.first().first().value<quint32>(), announced.first().last().value<quint32>(), &registry);
-    QSignalSpy outputChanged(output, &KWayland::Client::OutputDevice::changed);
+    KWayland::Client::OutputDevice *output =
+        registry.createOutputDevice(announced.first().first().value<quint32>(), announced.first().last().value<quint32>(), &registry);
+    QSignalSpy outputChanged(output, &KWayland::Client::OutputDevice::done);
     QVERIFY(outputChanged.isValid());
     wl_display_flush(m_connection->display());
-    if (outputChanged.isEmpty()) {
-        QVERIFY(outputChanged.wait());
-    }
+    QVERIFY(outputChanged.wait());
 
     QTEST(output->transform(), "expected");
 
     // change back to normal
     outputChanged.clear();
     m_serverOutputDevice->setTransform(OutputDeviceInterface::Transform::Normal);
-    if (outputChanged.isEmpty()) {
-        QVERIFY(outputChanged.wait());
-    }
+    QVERIFY(outputChanged.wait());
     QCOMPARE(output->transform(), OutputDevice::Transform::Normal);
 }
 
 void TestWaylandOutputDevice::testEnabled()
 {
     KWayland::Client::Registry registry;
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfacesAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
     QSignalSpy announced(&registry, &KWayland::Client::Registry::outputDeviceAnnounced);
+    registry.setEventQueue(m_queue);
     registry.create(m_connection->display());
     QVERIFY(registry.isValid());
     registry.setup();
     wl_display_flush(m_connection->display());
-    QVERIFY(announced.wait());
+    QVERIFY(interfacesAnnouncedSpy.wait());
 
     KWayland::Client::OutputDevice output;
-    QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::changed);
+    QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::done);
     QVERIFY(outputChanged.isValid());
     output.setup(registry.bindOutputDevice(announced.first().first().value<quint32>(), announced.first().last().value<quint32>()));
     wl_display_flush(m_connection->display());
@@ -463,31 +594,40 @@ void TestWaylandOutputDevice::testEnabled()
     QVERIFY(enabledChanged.isValid());
 
     m_serverOutputDevice->setEnabled(OutputDeviceInterface::Enablement::Disabled);
-    QVERIFY(enabledChanged.wait(200));
+    QVERIFY(enabledChanged.wait());
     QCOMPARE(output.enabled(), OutputDevice::Enablement::Disabled);
+    if (changed.count() != enabledChanged.count()) {
+        QVERIFY(changed.wait());
+    }
     QCOMPARE(changed.count(), enabledChanged.count());
 
     m_serverOutputDevice->setEnabled(OutputDeviceInterface::Enablement::Enabled);
-    QVERIFY(enabledChanged.wait(200));
+    QVERIFY(enabledChanged.wait());
     QCOMPARE(output.enabled(), OutputDevice::Enablement::Enabled);
+    if (changed.count() != enabledChanged.count()) {
+        QVERIFY(changed.wait());
+    }
     QCOMPARE(changed.count(), enabledChanged.count());
 }
 
 void TestWaylandOutputDevice::testEdid()
 {
     KWayland::Client::Registry registry;
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfacesAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
     QSignalSpy announced(&registry, &KWayland::Client::Registry::outputDeviceAnnounced);
+    registry.setEventQueue(m_queue);
     registry.create(m_connection->display());
     QVERIFY(registry.isValid());
     registry.setup();
     wl_display_flush(m_connection->display());
-    QVERIFY(announced.wait());
+    QVERIFY(interfacesAnnouncedSpy.wait());
 
     KWayland::Client::OutputDevice output;
 
     QCOMPARE(output.edid(), QByteArray());
 
-    QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::changed);
+    QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::done);
     QVERIFY(outputChanged.isValid());
     output.setup(registry.bindOutputDevice(announced.first().first().value<quint32>(), announced.first().last().value<quint32>()));
     wl_display_flush(m_connection->display());
@@ -498,15 +638,18 @@ void TestWaylandOutputDevice::testEdid()
 void TestWaylandOutputDevice::testId()
 {
     KWayland::Client::Registry registry;
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfacesAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
     QSignalSpy announced(&registry, &KWayland::Client::Registry::outputDeviceAnnounced);
+    registry.setEventQueue(m_queue);
     registry.create(m_connection->display());
     QVERIFY(registry.isValid());
     registry.setup();
     wl_display_flush(m_connection->display());
-    QVERIFY(announced.wait());
+    QVERIFY(interfacesAnnouncedSpy.wait());
 
     KWayland::Client::OutputDevice output;
-    QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::changed);
+    QSignalSpy outputChanged(&output, &KWayland::Client::OutputDevice::done);
     QVERIFY(outputChanged.isValid());
     output.setup(registry.bindOutputDevice(announced.first().first().value<quint32>(), announced.first().last().value<quint32>()));
     wl_display_flush(m_connection->display());
@@ -518,23 +661,30 @@ void TestWaylandOutputDevice::testId()
     QVERIFY(idChanged.isValid());
 
     m_serverOutputDevice->setUuid("42");
-    QVERIFY(idChanged.wait(200));
+    QVERIFY(idChanged.wait());
+    QCOMPARE(idChanged.first().first().toByteArray(), QByteArray("42"));
+    idChanged.clear();
     QCOMPARE(output.uuid(), QByteArray("42"));
 
     m_serverOutputDevice->setUuid("4711");
-    QVERIFY(idChanged.wait(200));
+    QVERIFY(idChanged.wait());
+    QCOMPARE(idChanged.first().first().toByteArray(), QByteArray("4711"));
+    idChanged.clear();
     QCOMPARE(output.uuid(), QByteArray("4711"));
 }
 
 void TestWaylandOutputDevice::testDone()
 {
     KWayland::Client::Registry registry;
+    QSignalSpy interfacesAnnouncedSpy(&registry, &KWayland::Client::Registry::interfacesAnnounced);
+    QVERIFY(interfacesAnnouncedSpy.isValid());
     QSignalSpy announced(&registry, &KWayland::Client::Registry::outputDeviceAnnounced);
+    registry.setEventQueue(m_queue);
     registry.create(m_connection->display());
     QVERIFY(registry.isValid());
     registry.setup();
     wl_display_flush(m_connection->display());
-    QVERIFY(announced.wait());
+    QVERIFY(interfacesAnnouncedSpy.wait());
 
     KWayland::Client::OutputDevice output;
     QSignalSpy outputDone(&output, &KWayland::Client::OutputDevice::done);
@@ -544,6 +694,5 @@ void TestWaylandOutputDevice::testDone()
     QVERIFY(outputDone.wait());
 }
 
-
 QTEST_GUILESS_MAIN(TestWaylandOutputDevice)
 #include "test_wayland_outputdevice.moc"
