@@ -1,44 +1,30 @@
-/********************************************************************
-Copyright 2014  Martin Gräßlin <mgraesslin@kde.org>
+/*
+    SPDX-FileCopyrightText: 2014 Martin Gräßlin <mgraesslin@kde.org>
 
-This library is free software; you can redistribute it and/or
-modify it under the terms of the GNU Lesser General Public
-License as published by the Free Software Foundation; either
-version 2.1 of the License, or (at your option) version 3, or any
-later version accepted by the membership of KDE e.V. (or its
-successor approved by the membership of KDE e.V.), which shall
-act as a proxy defined in Section 6 of version 3 of the license.
-
-This library is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public
-License along with this library.  If not, see <http://www.gnu.org/licenses/>.
-*********************************************************************/
+    SPDX-License-Identifier: LGPL-2.1-only OR LGPL-3.0-only OR LicenseRef-KDE-Accepted-LGPL
+*/
 #include "fullscreen_shell.h"
-#include "surface.h"
 #include "output.h"
+#include "surface.h"
 #include "wayland_pointer_p.h"
 
 #include <QDebug>
 // wayland
-#include <wayland-fullscreen-shell-client-protocol.h>
 #include <wayland-client-protocol.h>
+#include <wayland-fullscreen-shell-client-protocol.h>
 
 namespace KWayland
 {
 namespace Client
 {
-
-class FullscreenShell::Private
+class Q_DECL_HIDDEN FullscreenShell::Private
 {
 public:
     Private(FullscreenShell *q);
     void setup(_wl_fullscreen_shell *shell);
 
     WaylandPointer<_wl_fullscreen_shell, _wl_fullscreen_shell_release> shell;
+    EventQueue *queue = nullptr;
     bool capabilityArbitraryModes = false;
     bool capabilityCursorPlane = false;
 
@@ -49,9 +35,7 @@ private:
     FullscreenShell *q;
 };
 
-_wl_fullscreen_shell_listener FullscreenShell::Private::s_fullscreenShellListener = {
-    FullscreenShell::Private::capabilitiesAnnounce
-};
+_wl_fullscreen_shell_listener FullscreenShell::Private::s_fullscreenShellListener = {FullscreenShell::Private::capabilitiesAnnounce};
 
 FullscreenShell::Private::Private(FullscreenShell *q)
     : q(q)
@@ -68,7 +52,7 @@ void FullscreenShell::Private::setup(_wl_fullscreen_shell *s)
 
 void FullscreenShell::Private::capabilitiesAnnounce(void *data, _wl_fullscreen_shell *shell, uint32_t capability)
 {
-    auto s = reinterpret_cast<FullscreenShell::Private*>(data);
+    auto s = reinterpret_cast<FullscreenShell::Private *>(data);
     Q_ASSERT(shell == s->shell);
     s->handleCapabilities(capability);
 }
@@ -77,11 +61,11 @@ void FullscreenShell::Private::handleCapabilities(uint32_t capability)
 {
     if (capability & _WL_FULLSCREEN_SHELL_CAPABILITY_ARBITRARY_MODES) {
         capabilityArbitraryModes = true;
-        emit q->capabilityArbitraryModesChanged(capabilityArbitraryModes);
+        Q_EMIT q->capabilityArbitraryModesChanged(capabilityArbitraryModes);
     }
     if (capability & _WL_FULLSCREEN_SHELL_CAPABILITY_CURSOR_PLANE) {
         capabilityCursorPlane = true;
-        emit q->capabilityCursorPlaneChanged(capabilityCursorPlane);
+        Q_EMIT q->capabilityCursorPlaneChanged(capabilityCursorPlane);
     }
 }
 
@@ -111,6 +95,16 @@ void FullscreenShell::setup(_wl_fullscreen_shell *shell)
     d->setup(shell);
 }
 
+EventQueue *FullscreenShell::eventQueue() const
+{
+    return d->queue;
+}
+
+void FullscreenShell::setEventQueue(EventQueue *queue)
+{
+    d->queue = queue;
+}
+
 void FullscreenShell::present(wl_surface *surface, wl_output *output)
 {
     Q_ASSERT(d->shell);
@@ -141,3 +135,5 @@ bool FullscreenShell::hasCapabilityCursorPlane() const
 
 }
 }
+
+#include "moc_fullscreen_shell.cpp"
